use crate::core::*;

use super::{ctypeless::*, wrapped_origin::*};

impl<'a, Ctx: 'a + Context, A: Mentionable<'a, Ctx>> Point<'a, Ctx, A> {
    /// Typeless version of the point.
    pub fn typeless(&self) -> Point<'a, Ctx, TypelessMentionable<'a, Ctx>> {
        Point {
            point: self.point,
            origin: self.origin.clone().map(
                TypelessMentionable::from_typed,
                TypelessError::from_typed,
                TypelessFactory::from_typed,
            ),
        }
    }
}

pub trait MentionableExt2<'a, Ctx: 'a + Context>: Mentionable<'a, Ctx> {
    /// References ([Point]s) to other objects. Typeless.
    fn points_typeless(&self, points: &mut Vec<Point<'a, Ctx, TypelessMentionable<'a, Ctx>>>);
    /// [Vec] of [Point]s as used by [`Mentionable::topology`].
    fn points_vec_2(&self) -> Vec<Point<'a, Ctx, TypelessMentionable<'a, Ctx>>>;
}

impl<'a, Ctx: 'a + Context, A: Mentionable<'a, Ctx>> MentionableExt2<'a, Ctx> for A {
    fn points_typeless(&self, points: &mut Vec<Point<'a, Ctx, TypelessMentionable<'a, Ctx>>>) {
        self.points_typed(points)
    }

    fn points_vec_2(&self) -> Vec<Point<'a, Ctx, TypelessMentionable<'a, Ctx>>> {
        let mut points = Vec::new();
        self.points_typeless(&mut points);
        points
    }
}

impl<'a, Ctx: 'a + Context> TakesPoints<'a, Ctx>
    for Vec<Point<'a, Ctx, TypelessMentionable<'a, Ctx>>>
{
    fn take<A: Mentionable<'a, Ctx>>(&mut self, point: &Point<'a, Ctx, A>) {
        self.push(point.typeless());
    }
}
